﻿namespace DemoWebsocket.Server.WebSockets
{
    using System;
    using System.Collections.Concurrent;
    using System.Linq;
    using System.Net.WebSockets;
    using System.Threading;
    using System.Threading.Tasks;

    /// <summary>
    /// The connection manager.
    /// </summary>
    public class ConnectionManager
    {
        /// <summary>
        /// The sockets.
        /// </summary>
        private readonly ConcurrentDictionary<string, WebSocket> _sockets;

        /// <summary>
        /// Initializes a new instance of the <see cref="ConnectionManager"/> class.
        /// </summary>
        public ConnectionManager()
        {
            this._sockets = new ConcurrentDictionary<string, WebSocket>();
        }

        /// <summary>
        /// Adds the socket.
        /// </summary>
        /// <param name="socket">The socket.</param>
        public void AddSocket(WebSocket socket)
        {
            this._sockets.TryAdd(CreateConnectionId(), socket);
        }

        /// <summary>
        /// Gets all.
        /// </summary>
        /// <returns>
        /// The concurrent dictionary.
        /// </returns>
        public ConcurrentDictionary<string, WebSocket> GetAll()
        {
            return this._sockets;
        }

        /// <summary>
        /// Gets the identifier.
        /// </summary>
        /// <param name="socket">The socket.</param>
        /// <returns>
        /// The identifier.
        /// </returns>
        public string GetId(WebSocket socket)
        {
            return this._sockets.FirstOrDefault(p => p.Value == socket).Key;
        }

        /// <summary>
        /// Gets the socket by identifier.
        /// </summary>
        /// <param name="id">The identifier.</param>
        /// <returns>
        /// The web socket.
        /// </returns>
        public WebSocket GetSocketById(string id)
        {
            return this._sockets.FirstOrDefault(p => p.Key == id).Value;
        }

        /// <summary>
        /// Removes the socket.
        /// </summary>
        /// <param name="id">The identifier.</param>
        /// <returns>
        /// The async task.
        /// </returns>
        public async Task RemoveSocket(string id)
        {
            this._sockets.TryRemove(id, out var socket);

            if (socket == null)
            {
                return;
            }

            if (socket.State != WebSocketState.Open)
            {
                return;
            }

            await socket.CloseAsync(
                WebSocketCloseStatus.NormalClosure,
                "Closed by the ConnectionManager",
                CancellationToken.None);
        }

        /// <summary>
        /// Creates the connection identifier.
        /// </summary>
        /// <returns>
        /// The connection identifier.
        /// </returns>
        private static string CreateConnectionId()
        {
            return Guid.NewGuid().ToString();
        }
    }
}