﻿namespace DemoWebsocket.Server.WebSockets
{
    using Microsoft.AspNetCore.Builder;
    using Microsoft.Extensions.DependencyInjection;
    using System.Reflection;

    /// <summary>
    /// The startup extensions.
    /// </summary>
    public static class StartupExtensions
    {
        /// <summary>
        /// Configures the web sockets.
        /// </summary>
        /// <param name="serviceCollection">The service collection.</param>
        /// <param name="assembly">The assembly.</param>
        /// <returns>
        /// The service collection with the web sockets.
        /// </returns>
        public static IServiceCollection AddWebSockets(this IServiceCollection serviceCollection, Assembly assembly = null)
        {
            serviceCollection.AddTransient<ConnectionManager>();

            var entryAssembly = assembly ?? Assembly.GetEntryAssembly();

            if (entryAssembly == null)
            {
                return serviceCollection;
            }

            foreach (var type in entryAssembly.ExportedTypes)
            {
                if (type.GetTypeInfo().BaseType == typeof(WebSocketHandler))
                {
                    serviceCollection.AddSingleton(type);
                }
            }

            return serviceCollection;
        }

        /// <summary>
        /// Use the web sockets.
        /// </summary>
        /// <typeparam name="TWebSocketHandler">The type of the web socket handler.</typeparam>
        /// <param name="applicationBuilder">The application builder.</param>
        /// <param name="pathMatch">The path match.</param>
        /// <returns>
        /// The application builder with the web sockets.
        /// </returns>
        public static IApplicationBuilder UseWebSocket<TWebSocketHandler>(this IApplicationBuilder applicationBuilder, string pathMatch = "/ws")
            where TWebSocketHandler : WebSocketHandler
        {
            var serviceScopeFactory = applicationBuilder.ApplicationServices.GetRequiredService<IServiceScopeFactory>();
            var serviceProvider = serviceScopeFactory.CreateScope().ServiceProvider;

            applicationBuilder.UseWebSockets();

            // Get the handler
            var handler = serviceProvider.GetService<TWebSocketHandler>();

            if (handler == null)
            {
                throw new WebSocketHandlerException($"Unable to find the WebSocketHandler << {nameof(TWebSocketHandler)} >>");
            }

            applicationBuilder.Map(pathMatch, app => app.UseMiddleware<WebSocketManagerMiddleware>(handler));

            return applicationBuilder;
        }
    }
}