﻿namespace DemoWebsocket.Server.WebSockets
{
    using System;
    using System.Net.WebSockets;
    using System.Text;
    using System.Threading;
    using System.Threading.Tasks;

    /// <summary>
    /// The web socket handler.
    /// </summary>
    public abstract class WebSocketHandler
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="WebSocketHandler"/> class.
        /// </summary>
        /// <param name="webSocketConnectionManager">The web socket connection manager.</param>
        protected WebSocketHandler(ConnectionManager webSocketConnectionManager)
        {
            this.WebSocketConnectionManager = webSocketConnectionManager;
        }

        /// <summary>
        /// Gets or sets the web socket connection manager.
        /// </summary>
        protected ConnectionManager WebSocketConnectionManager { get; set; }

        /// <summary>
        /// Called when [connected].
        /// </summary>
        /// <param name="socket">The socket.</param>
        /// <returns>
        /// The async task.
        /// </returns>
        public virtual async Task OnConnected(WebSocket socket)
        {
            this.WebSocketConnectionManager.AddSocket(socket);
        }

        /// <summary>
        /// Called when [disconnected].
        /// </summary>
        /// <param name="socket">The socket.</param>
        /// <returns>
        /// The async task.
        /// </returns>
        public virtual async Task OnDisconnected(WebSocket socket)
        {
            await this.WebSocketConnectionManager.RemoveSocket(this.WebSocketConnectionManager.GetId(socket));
        }

        /// <summary>
        /// Receives the asynchronous.
        /// </summary>
        /// <param name="socket">The socket.</param>
        /// <param name="result">The result.</param>
        /// <param name="buffer">The buffer.</param>
        /// <param name="counter">The counter.</param>
        /// <returns>
        /// The task.
        /// </returns>
        public abstract Task ReceiveAsync(WebSocket socket, WebSocketReceiveResult result, byte[] buffer, int counter);

        /// <summary>
        /// Sends the message asynchronous.
        /// </summary>
        /// <param name="socket">The socket.</param>
        /// <param name="message">The message.</param>
        /// <returns>
        /// The async task.
        /// </returns>
        public async Task SendMessageAsync(WebSocket socket, string message)
        {
            if (socket.State != WebSocketState.Open)
            {
                return;
            }

            await socket.SendAsync(
                new ArraySegment<byte>(Encoding.ASCII.GetBytes(message), 0, message.Length),
                WebSocketMessageType.Text,
                true,
                CancellationToken.None);
        }

        /// <summary>
        /// Sends the message asynchronous.
        /// </summary>
        /// <param name="socketId">The socket identifier.</param>
        /// <param name="message">The message.</param>
        /// <returns>
        /// The async task.
        /// </returns>
        public async Task SendMessageAsync(string socketId, string message)
        {
            await this.SendMessageAsync(this.WebSocketConnectionManager.GetSocketById(socketId), message);
        }

        /// <summary>
        /// Sends the message to all asynchronous.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <returns>
        /// The async task.
        /// </returns>
        public async Task SendMessageToAllAsync(string message)
        {
            foreach (var pair in this.WebSocketConnectionManager.GetAll())
            {
                if (pair.Value.State == WebSocketState.Open)
                {
                    await this.SendMessageAsync(pair.Value, message);
                }
            }
        }
    }
}