﻿namespace DemoWebsocket.Server.WebSockets
{
    using Microsoft.AspNetCore.Http;
    using System;
    using System.Collections.Generic;
    using System.Net.WebSockets;
    using System.Threading;
    using System.Threading.Tasks;

    /// <summary>
    /// The web socket manager middleware.
    /// </summary>
    public class WebSocketManagerMiddleware
    {
        /// <summary>
        /// The buffer size.
        /// </summary>
        private const int BufferSize = 1024 * 4;

        /// <summary>
        /// The next.
        /// </summary>
        private readonly RequestDelegate _next;

        /// <summary>
        /// Gets or sets the web socket handler.
        /// </summary>
        private readonly WebSocketHandler _webSocketHandler;

        /// <summary>
        /// Initializes a new instance of the <see cref="WebSocketManagerMiddleware"/> class.
        /// </summary>
        /// <param name="next">The next.</param>
        /// <param name="webSocketHandler">The web socket handler.</param>
        public WebSocketManagerMiddleware(
            RequestDelegate next,
            WebSocketHandler webSocketHandler)
        {
            this._next = next;
            this._webSocketHandler = webSocketHandler;
        }

        /// <summary>
        /// Invokes the specified context.
        /// </summary>
        /// <param name="context">The context.</param>
        /// <returns>
        /// The async task.
        /// </returns>
        public async Task Invoke(HttpContext context)
        {
            if (context == null)
            {
                throw new ArgumentNullException(nameof(context));
            }

            if (!context.WebSockets.IsWebSocketRequest)
            {
                return;
            }

            var socket = await context.WebSockets.AcceptWebSocketAsync();
            await this._webSocketHandler.OnConnected(socket);

            await Receive(
                socket,
                async (result, buffer, counter) =>
                {
                    switch (result.MessageType)
                    {
                        case WebSocketMessageType.Text:
                            await this._webSocketHandler.ReceiveAsync(socket, result, buffer, counter);
                            return;

                        case WebSocketMessageType.Close:
                            await this._webSocketHandler.OnDisconnected(socket);
                            return;
                    }
                });
        }

        /// <summary>
        /// Receives the specified socket.
        /// </summary>
        /// <param name="socket">The socket.</param>
        /// <param name="handleMessage">The handle message.</param>
        /// <returns>
        /// The async task.
        /// </returns>
        private static async Task Receive(WebSocket socket, Action<WebSocketReceiveResult, byte[], int> handleMessage)
        {
            var buffer = new byte[BufferSize];

            while (socket.State == WebSocketState.Open)
            {
                var data = new List<byte>();

                var result = await socket.ReceiveAsync(new ArraySegment<byte>(buffer), CancellationToken.None);

                data.AddRange(buffer);
                var counter = result.Count;

                if (!result.EndOfMessage)
                {
                    var endOfMessage = false;

                    while (!endOfMessage)
                    {
                        buffer = new byte[BufferSize];

                        result = await socket.ReceiveAsync(new ArraySegment<byte>(buffer), CancellationToken.None);

                        data.AddRange(buffer);
                        counter += result.Count;
                        endOfMessage = result.EndOfMessage;
                    }
                }

                handleMessage(result, data.ToArray(), counter);
            }
        }
    }
}