﻿namespace MyBeautifulAdmin.Components
{
    using Microsoft.AspNetCore.Components;
    using Microsoft.JSInterop;
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using Towel;

    public partial class Minesweeper
    {
        public const int MineValue = -1;

        private int _height = 10;

        private double _mineRatio = .1d;

        private Random _random = new Random();

        private int _width = 10;

        [Parameter]
        public int Height
        {
            get => _height;
            set
            {
                if (this._height == value)
                {
                    return;
                }

                this._height = value;
                this.OnHeightChanged();
                this.StateHasChanged();
            }
        }

        [Inject]
        public IJSRuntime JSRuntime { get; set; }

        [Parameter]
        public RenderFragment LoseMessage { get; set; }

        [Parameter]
        public double MineRatio
        {
            get => _mineRatio;
            set
            {
                if (this._mineRatio == value)
                {
                    return;
                }

                this._mineRatio = value;
                this.OnMineRatioChanged();
                this.StateHasChanged();
            }
        }

        [Parameter]
        public int Width
        {
            get => _width;
            set
            {
                if (this._width == value)
                {
                    return;
                }

                this._width = value;
                this.OnWidthChanged();
                this.StateHasChanged();
            }
        }

        [Parameter]
        public RenderFragment WinMessage { get; set; }

        internal List<MinesweeperCellData> Board { get; set; }

        internal string BoardStyle { get; set; }

        internal int MineCount { get; set; }

        private string LoseMessageDisplay => LoseCheck() ? "block" : "none";
        private string WinMessageDisplay => WinCheck() ? "block" : "none";

        internal async void Reveal(int row, int column)
        {
            var item = Board.FirstOrDefault(w => w.X == row && w.Y == column);

            if (item == null)
            {
                Console.WriteLine($"Unable to found the cell with X: {row} && Y: {column}");
                return;
            }

            item.Revealed = true;
            if (item.Value == 0)
            {
                foreach (var (Row, Column) in AdjacentTiles(column, row))
                {
                    var adjacentTile = Board.FirstOrDefault(w => w.X == Row && w.Y == Column);

                    if (adjacentTile == null)
                    {
                        Console.WriteLine($"Unable to found the adjacent cell with X: {Row} && Y: {Column}");
                        continue;
                    }

                    if (!adjacentTile.Revealed)
                    {
                        Reveal(Row, Column);
                    }
                }
            }

            StateHasChanged();
        }

        internal void RevealAll()
        {
            for (int i = 0; i < Width; i++)
            {
                for (int j = 0; j < Height; j++)
                {
                    var item = Board.FirstOrDefault(w => w.X == i && w.Y == j);

                    if (item == null)
                    {
                        Console.WriteLine($"Unable to found the cell with X: {i} && Y: {j}");
                        continue;
                    }

                    item.Revealed = true;
                }
            }

            StateHasChanged();
        }

        private IEnumerable<(int Row, int Column)> AdjacentTiles(int column, int row)
        {
            //    A B C
            //    D + E
            //    F G H

            /* A */
            if (row > 0 && column > 0)
            {
                yield return (row - 1, column - 1);
            }

            /* B */
            if (row > 0)
            {
                yield return (row - 1, column);
            }

            /* C */
            if (row > 0 && column < Width - 1)
            {
                yield return (row - 1, column + 1);
            }

            /* D */
            if (column > 0)
            {
                yield return (row, column - 1);
            }

            /* E */
            if (column < Width - 1)
            {
                yield return (row, column + 1);
            }

            /* F */
            if (row < Height - 1 && column > 0)
            {
                yield return (row + 1, column - 1);
            }

            /* G */
            if (row < Height - 1)
            {
                yield return (row + 1, column);
            }

            /* H */
            if (row < Height - 1 && column < Width - 1)
            {
                yield return (row + 1, column + 1);
            }
        }

        private void CalculateMineCount()
        {
            MineCount = (int)(Width * Height * MineRatio);
        }

        private async void GenerateBoard()
        {
            var board = new List<MinesweeperCellData>();

            for (var i = 0; i < Width; ++i)
            {
                for (var j = 0; j < Height; ++j)
                {
                    board.Add(new MinesweeperCellData { X = i, Y = j });
                }
            }

            foreach (var m in _random.NextUnique(MineCount, 0, Width * Height))
            {
                var i = m / Width;
                var j = m % Width;

                var item = board.FirstOrDefault(w => w.X == i && w.Y == j);

                if (item == null)
                {
                    Console.WriteLine($"Unable to found the cell with X: {i} && Y: {j}");
                    continue;
                }

                item.Value = MineValue;
                foreach (var tile in AdjacentTiles(j, i))
                {
                    var adjacentTile = board.FirstOrDefault(w => w.X == tile.Row && w.Y == tile.Column);

                    if (adjacentTile == null)
                    {
                        Console.WriteLine($"Unable to found the adjacent cell with X: {i} && Y: {j}");
                        continue;
                    }

                    if (adjacentTile.Value != MineValue)
                    {
                        ++adjacentTile.Value;
                    }
                }
            }

            Board = board;

            // Use the Console.WriteLine (No display the content of the variable)
            Console.WriteLine("Board: ");
            Console.WriteLine(Board);

            // Use JSRuntime (Display the content of the variable)
            await JSRuntime.InvokeVoidAsync("console.log", Board);
        }

        private void GenerateBoardStyle()
        {
            BoardStyle = "grid-template-columns:";

            for (var i = 0; i < Width; ++i)
            {
                BoardStyle += " auto";
            }
        }

        private bool LoseCheck()
        {
            return Board.Any(w => w.Revealed && w.Value == MineValue);
        }

        private void NewGame()
        {
            GenerateBoard();
            StateHasChanged();
        }

        private void OnHeightChanged()
        {
            GenerateBoardStyle();
            CalculateMineCount();
            GenerateBoard();
        }

        private void OnMineRatioChanged()
        {
            CalculateMineCount();
            GenerateBoard();
        }

        private void OnWidthChanged()
        {
            GenerateBoardStyle();
            CalculateMineCount();
            GenerateBoard();
        }

        private bool WinCheck()
        {
            var revealCount = 0;
            for (var i = 0; i < Width; ++i)
            {
                for (var j = 0; j < Height; ++j)
                {
                    var item = Board.FirstOrDefault(w => w.X == i && w.Y == j);

                    if (item == null)
                    {
                        Console.WriteLine($"Unable to found the cell with X: {i} && Y: {j}");
                        continue;
                    }

                    if (item.Revealed)
                    {
                        revealCount++;
                    }
                }
            }

            return revealCount == Width * Height - MineCount;
        }
    }
}