---
sidebar_position: 2
sidebar_label: Créer fichier CI
title: Créer un fichier CI
---

Le CI de votre projet utilise Drone.

## Créer le fichier CI initial

A la racine de votre projet créez un nouveau fichier vide avec le nom `.drone.yml`.

Remplissez le fichier avec les données initiales d'un fichier CI :

```yml
kind: pipeline
type: docker
name: default
 
trigger:
  event:
    - push

steps:
```

Ce code permet :
* Définir le type de pipeline et son nom.
* Lancer le CI lorsqu'un push est validé sur le référentiel.
* Sous `steps`, le fichier yaml contient la liste des travaux.

### Ajouter la tâche de build

En premier pour un CI est la construction de votre projet, ajoutez le code ci-dessous dans le fichier CI.

```yml
  - name: build
    image: mcr.microsoft.com/dotnet/sdk:6.0
    commands:
      - cd Sources/
      - dotnet restore MySolution.sln
      - dotnet build MySolution.sln -c Release --no-restore
```

### Ajouter la tâche de test

La deuxième étape consiste à tester votre projet.

```yml
  - name: tests
    image: mcr.microsoft.com/dotnet/sdk:6.0
    commands:
      - cd Sources/
      - dotnet restore MySolution.sln
      - dotnet test MySolution.sln --no-restore
    depends_on: [build]
```

### Ajouter la tâche d'analyse

Un CI est également présent pour analyser votre code !

```yml
  - name: code-analysis
    image: hub.codefirst.iut.uca.fr/thomas.bellembois/codefirst-dronesonarplugin-dotnet6
    commands:
      - cd Sources/
      - dotnet restore MySolution.sln
      - dotnet sonarscanner begin /k:$REPO_NAME /d:sonar.host.url=$$$${PLUGIN_SONAR_HOST} /d:sonar.coverageReportPaths="coveragereport/SonarQube.xml" /d:sonar.coverage.exclusions="Tests/**" /d:sonar.login=$$$${PLUGIN_SONAR_TOKEN} 
      - dotnet build MySolution.sln -c Release --no-restore
      - dotnet test MySolution.sln --logger trx --no-restore /p:CollectCoverage=true /p:CoverletOutputFormat=cobertura --collect "XPlat Code Coverage"
      - reportgenerator -reports:"**/coverage.cobertura.xml" -reporttypes:SonarQube -targetdir:"coveragereport"
      - dotnet publish MySolution.sln -c Release --no-restore -o $CI_PROJECT_DIR/build/release
      - dotnet sonarscanner end /d:sonar.login=$$$${PLUGIN_SONAR_TOKEN}
    secrets: [ SECRET_SONAR_LOGIN ]
    settings:
      # accessible en ligne de commande par $${PLUGIN_SONAR_HOST}
      sonar_host: https://codefirst.iut.uca.fr/sonar/
      # accessible en ligne de commande par $${PLUGIN_SONAR_TOKEN}
      sonar_token:
        from_secret: SECRET_SONAR_LOGIN
    depends_on: [tests]
```

### Ajouter le travail de documentation

Votre projet contient également une documentation, créez un job pour la générer.

```yml
  - name: generate-and-deploy-docs
    image: hub.codefirst.iut.uca.fr/thomas.bellembois/codefirst-docdeployer
    failure: ignore
    volumes:
      - name: docs
        path: /docs
    commands:
      - /entrypoint.sh
    when:
      branch:
        - master
    depends_on: [ build ]

# The volumes declaration appear at the end of the file, after all steps
volumes:
- name: docs
  temp: {}
```

## Ajouter la tâche de CD

Pour le CD, nous déployons une image docker de votre projet, ajoutons le travail pour construire et déployons votre image docker dans un registre.

```yaml
  - name: docker-build
    image: plugins/docker
    settings:
       dockerfile: Sources/Dockerfile
       context: .
       registry: hub.codefirst.iut.uca.fr
       repo: hub.codefirst.iut.uca.fr/my-group/my-application-client
       username:
         from_secret: SECRET_REGISTRY_USERNAME
       password:
         from_secret: SECRET_REGISTRY_PASSWORD
    when:
      branch:
        - master
```

## Exemple de fichier complet

Découvrez ici un dossier complet avec tous les tâches.

```yaml
kind: pipeline
type: docker
name: default
 
trigger:
  event:
    - push

steps:
  - name: build
    image: mcr.microsoft.com/dotnet/sdk:6.0
    commands:
      - cd Sources/
      - dotnet restore MySolution.sln
      - dotnet build MySolution.sln -c Release --no-restore

  - name: tests
    image: mcr.microsoft.com/dotnet/sdk:6.0
    commands:
      - cd Sources/
      - dotnet restore MySolution.sln
      - dotnet test MySolution.sln --no-restore
    depends_on: [build]

  - name: code-analysis
    image: hub.codefirst.iut.uca.fr/thomas.bellembois/codefirst-dronesonarplugin-dotnet6
    commands:
      - cd Sources/
      - dotnet restore MySolution.sln
      - dotnet sonarscanner begin /k:$REPO_NAME /d:sonar.host.url=$$$${PLUGIN_SONAR_HOST} /d:sonar.coverageReportPaths="coveragereport/SonarQube.xml" /d:sonar.coverage.exclusions="Tests/**" /d:sonar.login=$$$${PLUGIN_SONAR_TOKEN} 
      - dotnet build MySolution.sln -c Release --no-restore
      - dotnet test MySolution.sln --logger trx --no-restore /p:CollectCoverage=true /p:CoverletOutputFormat=cobertura --collect "XPlat Code Coverage"
      - reportgenerator -reports:"**/coverage.cobertura.xml" -reporttypes:SonarQube -targetdir:"coveragereport"
      - dotnet publish MySolution.sln -c Release --no-restore -o $CI_PROJECT_DIR/build/release
      - dotnet sonarscanner end /d:sonar.login=$$$${PLUGIN_SONAR_TOKEN}
    secrets: [ SECRET_SONAR_LOGIN ]
    settings:
      # accessible en ligne de commande par $${PLUGIN_SONAR_HOST}
      sonar_host: https://codefirst.iut.uca.fr/sonar/
      # accessible en ligne de commande par $${PLUGIN_SONAR_TOKEN}
      sonar_token:
        from_secret: SECRET_SONAR_LOGIN
    depends_on: [tests]

  - name: generate-and-deploy-docs
    image: hub.codefirst.iut.uca.fr/thomas.bellembois/codefirst-docdeployer
    failure: ignore
    volumes:
      - name: docs
        path: /docs
    commands:
      - /entrypoint.sh
    when:
      branch:
        - master
    depends_on: [ build ]

  - name: docker-build
    image: plugins/docker
    settings:
       dockerfile: Sources/Dockerfile
       context: .
       registry: hub.codefirst.iut.uca.fr
       repo: hub.codefirst.iut.uca.fr/my-group/my-application-client
       username:
         from_secret: SECRET_REGISTRY_USERNAME
       password:
         from_secret: SECRET_REGISTRY_PASSWORD
    when:
      branch:
        - master

volumes:
- name: docs
  temp: {}
```





